'use babel';
'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _discordGraf = require('discord-graf');

var _diceExpressionEvaluator = require('dice-expression-evaluator');

var _diceExpressionEvaluator2 = _interopRequireDefault(_diceExpressionEvaluator);

var _commonTags = require('common-tags');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

Function.prototype.$asyncbind = function $asyncbind(self, catcher) {
	var resolver = this;

	if (catcher === true) {
		if (!Function.prototype.$asyncbind.EagerThenable) Function.prototype.$asyncbind.EagerThenable = function factory(tick) {
			var _tasks = [];

			if (!tick) {
				try {
					tick = process.nextTick;
				} catch (ex) {
					tick = function tick(p) {
						setTimeout(p, 0);
					};
				}
			}

			function _untask() {
				for (var i = 0; i < _tasks.length; i += 2) {
					var t = _tasks[i + 1],
					    r = _tasks[i];

					for (var j = 0; j < t.length; j++) t[j].call(null, r);
				}

				_tasks = [];
			}

			function isThenable(obj) {
				return obj && obj instanceof Object && typeof obj.then === 'function';
			}

			function EagerThenable(resolver) {
				function done(inline) {
					var w;
					if (_sync || phase < 0 || (w = _thens[phase]).length === 0) return;

					_tasks.push(result, w);

					_thens = [[], []];
					if (_tasks.length === 2) inline ? _untask() : tick(_untask);
				}

				function resolveThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 0;
					result = x;
					done(true);
				}

				function rejectThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 1;
					result = x;
					done(true);
				}

				function settler(resolver, rejecter) {
					_thens[0].push(resolver);

					_thens[1].push(rejecter);

					done();
				}

				function toString() {
					return 'EagerThenable{' + {
						'-1': 'pending',
						0: 'resolved',
						1: 'rejected'
					}[phase] + '}=' + result.toString();
				}

				this.then = settler;
				this.toString = toString;
				var _thens = [[], []],
				    _sync = true,
				    phase = -1,
				    result;
				resolver.call(null, resolveThen, rejectThen);
				_sync = false;
				done();
			}

			EagerThenable.resolve = function (v) {
				return isThenable(v) ? v : {
					then: function then(resolve, reject) {
						return resolve(v);
					}
				};
			};

			return EagerThenable;
		}();
		return new Function.prototype.$asyncbind.EagerThenable(boundThen);
	}

	if (catcher) {
		if (Function.prototype.$asyncbind.wrapAsyncStack) catcher = Function.prototype.$asyncbind.wrapAsyncStack(catcher);
		return then;
	}

	function then(result, error) {
		try {
			return result && result instanceof Object && typeof result.then === 'function' ? result.then(then, catcher) : resolver.call(self, result, error || catcher);
		} catch (ex) {
			return (error || catcher)(ex);
		}
	}

	function boundThen(result, error) {
		return resolver.call(self, result, error);
	}

	boundThen.then = boundThen;
	return boundThen;
};

const pattern = /^(.+?)(?:(>{1,2}|<{1,2})\s*([0-9]+?))?\s*$/;

class RollDiceCommand extends _discordGraf.Command {
	constructor(bot) {
		super(bot, {
			name: 'roll',
			aliases: ['dice', 'roll-dice', 'dice-roll', '(roll: xxxx)'],
			module: 'dice',
			memberName: 'roll',
			description: 'Rolls specified dice.',
			usage: 'roll [dice expression]',
			details: _commonTags.oneLine`
				Dice expressions can contain the standard representations of dice in text form (e.g. 2d20 is two 20-sided dice), with addition and subtraction allowed.
				You may also use a single \`>\` or \`<\` symbol at the end of the expression to add a target for the total dice roll - for example, \`2d20 + d15 > 35\`.
				You can count the number of successes using \`>>\` or \`<<\`, but only on a single dice expression - for example, \`4d30 >> 20\`.
				When running the command with no dice expression, it will default to a D20.
				When just a single plain number is provided, it will be interpreted as a single die with that many sides.
			`,
			examples: ['roll 2d20', 'roll 3d20 - d10 + 6', 'roll d20 > 10', 'roll', 'roll 30', 'Billy McBillface attempts to slay the dragon. (Roll: d20 > 10)'],
			patterns: [/\(\s*(?:roll|dice|rolldice|diceroll):\s*(.+?)(?:(>{1,2}|<{1,2})\s*([0-9]+?))?\s*\)/i]
		});
	}

	run(message, args, fromPattern) {
		return new Promise(function ($return, $error) {
			// eslint-disable-line complexity
			const firstArgIndex = fromPattern ? 1 : 0;
			if (!args[firstArgIndex]) {
				args[firstArgIndex] = 'd20';
			} else {
				const rawNumber = parseInt(args[firstArgIndex]);
				if (!isNaN(rawNumber) && String(rawNumber) === args[firstArgIndex]) args[firstArgIndex] = `d${ rawNumber }`;
			}

			try {
				const matches = fromPattern ? args : pattern.exec(args[0]);
				const dice = new _diceExpressionEvaluator2.default(matches[1]);

				// Restrict the maximum dice count
				const totalDice = dice.dice.reduce((prev, die) => prev + (die.diceCount || 1), 0);
				if (totalDice > 1000) return $return({ plain: `${ message.author } might hurt themselves by rolling that many dice at once!` });

				// Roll the dice
				const rollResult = dice.roll();
				this.bot.logger.debug('Dice rolled', { dice: dice.dice, result: rollResult, totalDice: totalDice });

				if (matches[2]) {
					// Deal with target operations
					const target = parseInt(matches[3]);
					let response;

					// Target for total roll
					if (matches[2] === '>' || matches[2] === '<') {
						const success = matches[2] === '>' ? rollResult.roll > target : rollResult.roll < target;
						const diceList = this.buildDiceList(rollResult, totalDice);
						response = _commonTags.oneLine`
						${ message.author } has **${ success ? 'succeeded' : 'failed' }**.
						(Rolled ${ rollResult.roll }, ${ !success ? 'not' : '' } ${ matches[2] === '>' ? 'greater' : 'less' } than ${ target }${ diceList ? `;   ${ diceList }` : '' })
					`;

						// Target for individual dice (success counting)
					} else if (matches[2] === '>>' || matches[2] === '<<') {
						if (rollResult.diceRaw.length !== 1) return $return({ plain: `${ message.author } tried to count successes with multiple dice expressions.` });
						const successes = rollResult.diceRaw[0].reduce((prev, die) => prev + (matches[2] === '>>' ? die > target : die < target), 0);
						response = _commonTags.oneLine`
						${ message.author } has **${ successes > 0 ? `succeeded ${ successes } time${ successes !== 1 ? 's' : '' }` : `failed` }**.
						${ rollResult.diceRaw[0].length > 1 ? `(${ rollResult.diceRaw[0].join(',   ') })` : '' }
					`;

						// Oh dear.
					} else {
						return $error(new Error('Unknown target operator. This should not ever happen.'));
					}

					return $return({ plain: response, editable: false });
				} else {
					const diceList = this.buildDiceList(rollResult, totalDice);
					return $return({
						plain: `${ message.author } rolled **${ rollResult.roll }**.${ diceList ? ` (${ diceList })` : '' }`,
						editable: false
					});
				}
			} catch (err) {
				return $return({ plain: `${ message.author } specified an invalid dice expression.` });
			}
			return $return();
		}.$asyncbind(this));
	}

	buildDiceList(result, totalDice) {
		let diceList = '';
		if (totalDice <= 100 && (result.diceRaw.length > 1 || result.diceRaw.length > 0 && result.diceRaw[0].length > 1)) {
			diceList = result.diceRaw.map((res, i) => this.bot.util.nbsp(res.length > 1 ? `${ res.join(' + ') } = ${ result.diceSums[i] }` : res[0])).join(',   ');
		}
		return diceList;
	}
}
exports.default = RollDiceCommand;
//# sourceMappingURL=roll.js.map
